@@ -6,13 +6,11 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # =======================================================================
 
-"""mapper objects"""
+"""sampler objects"""
 
-__all__ = ['BaseIntegrator', 'Integrator', 'SunSkyIntegrator',
-           'MetricSet', 'PositionIndex', 'retina']
+__all__ = ["Integrator", "IntegratorDS", "ZonalIntegrator", "ZonalIntegratorDS"]
 
-from raytraverse.integrator.baseintegrator import BaseIntegrator
 from raytraverse.integrator.integrator import Integrator
-from raytraverse.integrator.sunskyintegrator import SunSkyIntegrator
-from raytraverse.integrator.metricset import MetricSet
-from raytraverse.integrator.positionindex import PositionIndex
+from raytraverse.integrator.integratords import IntegratorDS
+from raytraverse.integrator.zonalintegrator import ZonalIntegrator
+from raytraverse.integrator.zonalintegratords import ZonalIntegratorDS
