@@ -34,12 +34,29 @@ class PlanMapper(Mapper):
         positive Z rotation for point grid alignment
     zheight: float, optional
         override calculated zheight
-    name str, optional
+    name: str, optional
         plan mapper name used for output file naming
+    jitterrate: float, optional
+        proportion of cell to jitter within
+    autorotate: bool, optional
+        if true set rotation based on long axis of area geometry
+    autogrid: int, optional
+        if given, autoset ptres based on this minimum number of points at
+        level 0 along the minimum dimemsion (width or height)
     """
 
     def __init__(self, area, ptres=1.0, rotation=0.0, zheight=None,
-                 name="plan"):
+                 name="plan", jitterrate=0.5, autorotate=False, autogrid=None):
+        if autorotate:
+            rotation = self.__class__(area, autorotate=False,
+                                      autogrid=None)._auto_rotate()
+        if autogrid is not None:
+            pm = self.__class__(area, autorotate=False, autogrid=None)
+            hv = pm.bbox[1] - pm.bbox[0]
+            if hv[0] > hv[1]:
+                ptres = hv[1]/(autogrid - 1) - 1e-3
+            else:
+                ptres = hv[0]/(autogrid - 1) - 1e-3
         #: float: ccw rotation (in degrees) for point grid on plane
         self.rotation = rotation
         #: float: point resolution for area look ups and grid
@@ -47,18 +64,20 @@ class PlanMapper(Mapper):
         self._bbox = None
         self._zheight = zheight
         self._path = []
-        self.update_bbox(area, updatez=zheight is None)
-        super().__init__(name=name, sf=self._sf, bbox=self.bbox)
+        self._candidates = None
+        xyz = self.update_bbox(area, updatez=zheight is None)
+        super().__init__(dxyz=xyz, name=name, sf=self._sf, bbox=self.bbox,
+                         jitterrate=jitterrate)
 
     @property
     def dxyz(self):
-        """(float, float, float) central view direction"""
+        """(float, float, float) central view point"""
         return self._dxyz
 
     @dxyz.setter
     def dxyz(self, xyz):
         """set view parameters"""
-        self._dxyz = translate.norm1(np.asarray(xyz).ravel()[0:3])
+        self._dxyz = np.asarray(xyz).ravel()[0:3]
 
     @property
     def rotation(self):
@@ -96,12 +115,14 @@ class PlanMapper(Mapper):
         self._bbox = bbox
         if updatez:
             self._zheight = z
+        xyz = np.concatenate((np.average(self.bbox, 0), [self._zheight]))
         self._sf = self.bbox[1] - self.bbox[0]
         self._path = []
         for pt in paths:
             p = (np.concatenate((pt, [pt[0]])) - bbox[0, 0:2])/self._sf
             xy = Path(p, closed=True)
             self._path.append(xy)
+        return xyz
 
     def uv2xyz(self, uv, stackorigin=False):
         """transform from mapper UV space to world xyz"""
@@ -157,7 +178,7 @@ class PlanMapper(Mapper):
     def shape(self, level=0):
         return np.ceil(self._sf/self.ptres - 1e-4).astype(int)*2**level
 
-    def point_grid(self, jitter=True, level=0, masked=True):
+    def point_grid(self, jitter=True, level=0, masked=True, snap=None):
         """generate a grid of points
 
         Parameters
@@ -169,15 +190,17 @@ class PlanMapper(Mapper):
             sets the resolution of the grid as a power of 2 from ptres
         masked: bool, optional
             apply in_view to points before returning
+        snap: int, optional
+            level to snap samples to when jitter=False should be > level
 
         Returns
         -------
         np.array
             shape (N, 3)
         """
-        return self.uv2xyz(self.point_grid_uv(jitter, level, masked))
+        return self.uv2xyz(self.point_grid_uv(jitter, level, masked, snap))
 
-    def point_grid_uv(self, jitter=True, level=0, masked=True):
+    def point_grid_uv(self, jitter=True, level=0, masked=True, snap=None):
         """add a grid of UV coordinates
 
         Parameters
@@ -189,20 +212,46 @@ class PlanMapper(Mapper):
             sets the resolution of the grid as a power of 2 from ptres
         masked: bool, optional
             apply in_view to points before returning
+        snap: int, optional
+            level to snap samples to when jitter=False should be > level
 
         Returns
         -------
         np.array
             shape (N, 2)
         """
+        # bypasses grid when initialized with points
         shape = self.shape(level)
         idx = np.arange(np.product(shape))
         uv = self.idx2uv(idx, shape, jitter)
+        if level == 0 and not jitter and self._candidates is not None:
+            uvup = self.xyz2uv(self._candidates)
+            uv[:] = self.bbox[1] * 2
+            uv[self.uv2idx(uvup, shape)] = uvup
+        elif not jitter and snap is not None and snap > level:
+            s2 = self.shape(snap)
+            uv -= .5 / s2
+            uv += np.random.default_rng().integers(0, 2, uv.shape)/s2
         if masked:
             return uv[self.in_view_uv(uv, False)]
         else:
             return uv
 
+    def _auto_rotate(self):
+        v = np.vstack(self.borders())
+        left = np.average(v[np.isclose(self.bbox[0, 0], v[:, 0])], 0)
+        right = np.average(v[np.isclose(self.bbox[1, 0], v[:, 0])], 0)
+        top = np.average(v[np.isclose(self.bbox[1, 1], v[:, 1])], 0)
+        bottom = np.average(v[np.isclose(self.bbox[0, 1], v[:, 1])], 0)
+        vaxis = np.average((top - right, left - bottom), 0)
+        haxis = np.average((right - bottom, top - left), 0)
+        hv = np.linalg.norm((haxis, vaxis), axis=1)
+        if hv[0] > hv[1]:
+            rotation = np.arccos(np.dot(haxis/hv[0], (1, 0, 0)))*180/np.pi
+        else:
+            rotation = np.arccos(np.dot(vaxis/hv[1], (0, 1, 0)))*180/np.pi
+        return rotation
+
     def _rad_scene_to_paths(self, plane):
         """reads a radiance scene for polygons, and sets bounding paths
         zheight of plan (unless PlanMapper initialized with a zheight) will be
@@ -231,19 +280,21 @@ class PlanMapper(Mapper):
                   if not bool(re.match(r'#', i))]
             rad = " ".join(dl).split()
         pgs = [i for i, x in enumerate(rad) if x == "polygon"]
-        z = -1e10
         paths = []
+        zs = []
         for pg in pgs:
             npt = int(int(rad[pg + 4])/3)
             pt = np.reshape([i for i in rad[pg + 5:pg + 5 + npt*3]],
                             (npt, 3)).astype(float)
-            z = max(z, max(pt[:, 2]))
+            zs.append(pt[:, 2])
             pt2 = self.world2view(pt)
             paths.append(pt2[:, 0:2])
+        z = np.median(zs)
         return np.array(paths), z
 
     def _calc_border(self, points, level=0):
         """generate a border from convex hull of points"""
+        self._candidates = points
         o = self.ptres/2**(level + 1)
         try:
             hull = ConvexHull(points[:, 0:2])
