@@ -27,7 +27,7 @@ class ViewMapper(AngularMixin, Mapper):
     """
 
     def __init__(self, dxyz=(0.0, 1.0, 0.0), viewangle=360.0, name='view',
-                 origin=(0, 0, 0)):
+                 origin=(0, 0, 0), jitterrate=0.9):
         self._viewangle = viewangle
         if viewangle > 180:
             aspect = 2
@@ -39,7 +39,7 @@ class ViewMapper(AngularMixin, Mapper):
             sf = np.array((self._viewangle/180, self._viewangle/180))
             bbox = np.stack((.5 - sf/2, .5 + sf/2))
         super().__init__(dxyz=dxyz, sf=sf, bbox=bbox, aspect=aspect, name=name,
-                         origin=origin)
+                         origin=origin, jitterrate=jitterrate)
 
     @property
     def aspect(self):
