@@ -65,6 +65,8 @@ class RadianceRenderer(Renderer):
         if nproc is None:
             nproc = cls.nproc
         nproc = io.get_nproc(nproc)
+        if "-ab 0" in args:
+            nproc = 1
         cls.nproc = nproc
         cls.args = args
         cls._args = shlex.split(f"{cls.name} -n {nproc} {args}")
