@@ -5,10 +5,9 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # =======================================================================
-from glob import glob
-
 import numpy as np
 from clasp.script_tools import try_mkdir, sglob
+from scipy.spatial import cKDTree
 
 from raytraverse import io, translate
 from raytraverse.mapper import MaskedPlanMapper
@@ -16,7 +15,7 @@ from raytraverse.sampler import draw
 from raytraverse.sampler.samplerarea import SamplerArea
 from raytraverse.sampler.basesampler import BaseSampler, filterdict
 from raytraverse.sampler.sunsamplerpt import SunSamplerPt
-from raytraverse.lightfield import DayLightPlaneKD, LightPlaneKD
+from raytraverse.lightfield import SunsPlaneKD, LightPlaneKD
 from raytraverse.evaluate import SamplingMetrics
 from raytraverse.utility import pool_call
 
@@ -48,7 +47,7 @@ class SamplerSuns(BaseSampler):
     #: initial sampling threshold coefficient
     t0 = .05
     #: final sampling threshold coefficient
-    t1 = .5
+    t1 = .125
     #: upper bound for drawing from pdf
     ub = 8
 
@@ -56,6 +55,7 @@ class SamplerSuns(BaseSampler):
                  ptkwargs=None, areakwargs=None,
                  metricset=('avglum', 'loggcr')):
         super().__init__(scene, engine, accuracy, stype='sunpositions')
+        self._gcrnorm = 8
         if areakwargs is None:
             areakwargs = {}
         if ptkwargs is None:
@@ -159,9 +159,8 @@ class SamplerSuns(BaseSampler):
         self._areaweights = None
         levels = self.sampling_scheme(skymapper)
         super().run(skymapper, areamapper.name, levels, **kwargs)
-        return DayLightPlaneKD(self.scene, self.vecs, self._areamapper,
-                               f"{self._skymapper.name}_sun",
-                               includesky=self._specguide is not None)
+        return SunsPlaneKD(self.scene, self.vecs, self._areamapper,
+                           f"{self._skymapper.name}_sun")
 
     def _init4run(self, levels, **kwargs):
         """(re)initialize object for new run, ensuring properties are cleared
@@ -190,9 +189,11 @@ class SamplerSuns(BaseSampler):
         shape = self.levels[i]
         uv = self._skymapper.xyz2uv(vecs)
         idx = self._skymapper.uv2idx(uv, shape)
-        candidates = np.full((np.prod(shape), 2), 3.0)
+        candidates = self._skymapper.solar_grid_uv(jitter=self.jitter,
+                                                   level=i, masked=False)
         candidates[idx] = uv
-        self._lift_weights(i, candidates=candidates)
+        self._candidates = candidates
+        self._lift_weights(i)
         draws, p = self.draw(i)
         si, uv = self.sample_to_uv(draws, shape)
         if si.size > 0:
@@ -283,7 +284,9 @@ class SamplerSuns(BaseSampler):
         if len(pdraws) == 0:
             return np.empty(0), np.empty(0)
         si = np.stack(np.unravel_index(pdraws, shape))
-        return si, self._candidates[pdraws]
+        uv = self._candidates[pdraws]
+        self._candidates = None
+        return si, uv
 
     def _sample_sun(self, suni, sunpos, adraws):
         """this function is for calling with a process pool, by declaring the
@@ -312,7 +315,8 @@ class SamplerSuns(BaseSampler):
                 except KeyError:
                     mc = SamplingMetrics
                 areasampler.lum = lf.evaluate(1, metrics=metrics,
-                                              metricclass=mc)
+                                              metricclass=mc,
+                                              gcrnorm=self._gcrnorm)
                 shp = (*areasampler.sampling_scheme(amapper)[-1],
                        areasampler.features)
             except (ValueError, OSError):
@@ -362,7 +366,8 @@ class SamplerSuns(BaseSampler):
         else:
             cap = None
         lums = pool_call(self._sample_sun, list(zip(range(*idx), vecs,
-                         self._areadraws)), desc=level_desc, cap=cap)
+                         self._areadraws)), desc=level_desc, cap=cap,
+                         pbar=self.scene.dolog)
         lums = np.array(lums)
         # initialize areaweights here now that we know the shape
         if len(self.lum) == 0:
@@ -379,11 +384,10 @@ class SamplerSuns(BaseSampler):
         update = self._areaweights[..., si[0], si[1]]
         self._areaweights[..., si[0], si[1]] = np.where(lum > 0, lum, update)
 
-    def _lift_weights(self, i, candidates=None):
-        if candidates is None:
-            candidates = self._skymapper.solar_grid_uv(jitter=self.jitter,
-                                                       level=i, masked=False)
-        self._candidates = candidates
+    def _lift_weights(self, i):
+        if self._candidates is None:
+            self._candidates = self._skymapper.solar_grid_uv(jitter=self.jitter,
+                                                             level=i, masked=False)
         shape = self._skymapper.shape(i)
         mask = self._skymapper.in_solarbounds_uv(self._candidates,
                                                  level=i).reshape(shape)
