@@ -117,9 +117,6 @@ class SunSamplerPt(SamplerPt):
         vm: raytraverse.mapper.ViewMappper
         """
         if specguide is None:
-            self.scene.log(self, "Warning: no specular guide for "
-                                 f"{self.stype} sampling", err=True,
-                           level=self._slevel)
             self._specguide = None
         else:
             if not hasattr(specguide, '__len__'):
@@ -127,7 +124,7 @@ class SunSamplerPt(SamplerPt):
             details = []
             for skykd in specguide:
                 side = int(np.sqrt(skykd.srcn - 1))
-                skybin = translate.xyz2skybin(self.sunpos, side, tol=.125)
+                skybin = translate.xyz2skybin(self.sunpos, side, tol=.25/side)
                 shp = self.levels[self.specidx]
                 si = np.stack(np.unravel_index(np.arange(np.product(shp)), shp))
                 uv = (si.T + .5)/shp[1]
