@@ -30,6 +30,9 @@ class SkyData(object):
         (to correct model north, equivalent to clockwise rotation of scene)
     ground_fac: float, optional
         ground reflectance
+    intersky: bool, optional
+        include interreflection between ground and sky (mimics perezlum.cal,
+        not present in gendaymtx)
     skyres: float, optional
         approximate square patch size in degrees
     minalt: float, optional
@@ -38,9 +41,17 @@ class SkyData(object):
         minumum diffuse horizontal irradiance for daylight masking
     """
 
-    def __init__(self, wea, loc=None, skyro=0.0, ground_fac=0.15,
-                 skyres=10.0, minalt=2.0, mindiff=5.0):
+    def __init__(self, wea, loc=None, skyro=0.0, ground_fac=0.2, intersky=True,
+                 skyres=12.0, minalt=2.0, mindiff=5.0, mindir=0.0):
         self.skyres = skyres
+        self.intersky = intersky
+        if wea is None:
+            ground_fac = 1
+            minalt = 0
+            mindiff = 0
+            mindir = 0
+            skyro = 0
+            loc = None
         self.ground_fac = ground_fac
         if loc is None and wea is not None:
             try:
@@ -51,6 +62,7 @@ class SkyData(object):
         self._loc = loc
         self._minalt = minalt
         self._mindiff = mindiff
+        self._mindir = mindir
         self._skyro = skyro
         self._sunproxy = None
         self.skydata = wea
@@ -60,9 +72,14 @@ class SkyData(object):
     def skyres(self):
         return self._skyres
 
+    @property
+    def side(self):
+        return self._side
+
     @skyres.setter
     def skyres(self, s):
-        self._skyres = int(np.floor(90/s)*2)
+        self._side = int(np.floor(90/s)*2)
+        self._skyres = s
 
     @property
     def skyro(self):
@@ -79,6 +96,11 @@ class SkyData(object):
         """sun position and dirnorm diffhoriz"""
         return self._skydata
 
+    @property
+    def rowlabel(self):
+        """m,d,h (if known)"""
+        return self._rowlabel
+
     @skydata.setter
     def skydata(self, wea):
         """calculate sky matrix data and store in self._skydata
@@ -95,26 +117,37 @@ class SkyData(object):
             npz file will rewrite skyres
         """
         npzdata = self._load(wea)
+        rowlabel = None
         if npzdata:
-            skydata, smtx, sun, daymask = npzdata
-        else:
-            skydata, md, td = self.format_skydata(wea)
+            skydata, smtx, sun, daymask, rowlabel = npzdata
+        elif wea is not None:
+            skydata, md, td, rowlabel = self.format_skydata(wea)
             minz = np.sin(self._minalt * np.pi / 180)
             daymask = np.logical_and(skydata[:, 2] > minz,
                                      skydata[:, 4] > self._mindiff)
+            daymask = np.logical_and(daymask, skydata[:, 3] >= self._mindir)
             sxyz = skydata[daymask, 0:3]
             dirdif = skydata[daymask, 3:]
             if md is not None:
                 md = md[daymask]
             if hasattr(td, "__len__"):
                 td = td[daymask]
-            smtx, grnd, sun = skycalc.sky_mtx(sxyz, dirdif, self.skyres, md=md,
-                                              ground_fac=self.ground_fac, td=td)
+            smtx, grnd, sun = skycalc.sky_mtx(sxyz, dirdif, self.side, md=md,
+                                              ground_fac=self.ground_fac, td=td,
+                                              intersky=self.intersky)
             # ratio between actual solar disc and patch
-            omegar = np.square(0.2665 * np.pi * self.skyres / 180) * .5
+            omegar = np.square(0.2665 * np.pi * self.side / 180) * .5
             plum = sun[:, -1] * omegar
             sun = np.hstack((sun, plum[:, None]))
             smtx = np.hstack((smtx, grnd[:, None]))
+        else:
+            smtx = np.ones((1, self.side**2+1))
+            sun = np.array([[0, 0, 1, 0, 0]])
+            daymask = np.array([True])
+            skydata = np.array([[0, 0, 1, 0, 1]])
+        if rowlabel is None:
+            rowlabel = np.arange(skydata.shape[0]).reshape(-1, 1)
+        self._rowlabel = rowlabel
         self._smtx = smtx
         self._sun = sun
         self._daymask = daymask
@@ -127,7 +160,11 @@ class SkyData(object):
         if scene is not None:
             file = f"{scene.outdir}/{file}"
         kws = dict(skydata=self._skydata, smtx=self._smtx, sun=self._sun,
-                   daymask=self._daymask, loc=self._loc)
+                   daymask=self._daymask)
+        if self.loc is not None:
+            kws["loc"] = self._loc
+        if self.rowlabel.shape[1] > 1:
+            kws["rowlabel"] = self._rowlabel
         if compressed:
             np.savez_compressed(file, **kws)
         else:
@@ -143,10 +180,19 @@ class SkyData(object):
             smtx = result['smtx']
             sun = result['sun']
             daymask = result['daymask']
-            loc = result['loc']
-            self._loc = (loc[0], loc[1], int(loc[2]))
-            self._skyres = int(np.sqrt(smtx.shape[1] - 1))
-            return skydata, smtx, sun, daymask
+            try:
+                loc = result['loc']
+            except (ValueError, KeyError):
+                self._loc = None
+            else:
+                self._loc = (loc[0], loc[1], int(loc[2]))
+            try:
+                rowlabel = result['rowlabel']
+            except (ValueError, KeyError):
+                rowlabel = None
+            self._side = int(np.sqrt(smtx.shape[1] - 1))
+            self._skyres = 180 / self._side
+            return skydata, smtx, sun, daymask, rowlabel
 
     def format_skydata(self, dat):
         """process dat argument as skydata
@@ -161,6 +207,7 @@ class SkyData(object):
         loc = self._loc
         md = None
         td = 10.9735311509
+        rowlabel = None
         if hasattr(dat, 'shape'):
             skydat = dat
         else:
@@ -185,13 +232,14 @@ class SkyData(object):
                 times = skycalc.row_2_datetime64(skydat[:, 0:3])
                 xyz = skycalc.sunpos_xyz(times, *loc, ro=self._skyro)
                 md = skydat[:, 0:2].astype(int)
+                rowlabel = skydat[:, 0:3]
                 skydat = np.hstack((xyz, skydat[:, 3:]))
         else:
             raise ValueError('input data should be one of the following:'
                              '\n4 col: alt, az, dir, diff'
                              '\n5 col: dx, dy, dz, dir, diff'
                              '\n5 col: m, d, h, dir, diff')
-        return skydat, md, td
+        return skydat, md, td, rowlabel
 
     @property
     def daysteps(self):
@@ -238,7 +286,7 @@ class SkyData(object):
 
     @property
     def smtx(self):
-        """shape (np.sum(daymask), skyres**2 + 1) coefficients for each sky
+        """shape (np.sum(daymask), side**2 + 1) coefficients for each sky
         patch each row is a timestep, coefficients exclude sun"""
         return self._smtx[self.mask]
 
@@ -256,16 +304,19 @@ class SkyData(object):
 
     @sunproxy.setter
     def sunproxy(self, sxyz):
-        self._sunproxy = translate.xyz2skybin(sxyz, self.skyres)
+        self._sunproxy = translate.xyz2skybin(sxyz, self.side)
 
-    def smtx_patch_sun(self):
+    def smtx_patch_sun(self, includesky=True):
         """generate smtx with solar energy applied to proxy patch
-        for directly applying to skysampler data (without direct sun components
+        for directly applying to skysampler data (without direct sun components)
         can also be used in a partial mode (with sun view / without sun
-        reflection."""
-        wsun = np.copy(self.smtx)
-        r = range(wsun.shape[0])
-        wsun[range(wsun.shape[0]), self.sunproxy] += self.sun[r, 4]
+        reflection.)"""
+        if includesky:
+            wsun = np.copy(self.smtx)
+        else:
+            wsun = np.zeros_like(self.smtx)
+        r = wsun.shape[0]
+        wsun[range(r), self.sunproxy] += self.sun[:, 4]
         return wsun
 
     def header(self):
@@ -277,7 +328,7 @@ class SkyData(object):
             hdr = "LOCATION= None"
         return hdr
 
-    def fill_data(self, x, fill_value=0.0):
+    def fill_data(self, x, fill_value=0.0, rowlabels=False):
         """
         Parameters
         ----------
@@ -285,24 +336,34 @@ class SkyData(object):
             first axis size = len(self.daymask[self.mask])
         fill_value: Union[int, float], optional
             value in padded array
+        rowlabels: bool, optional
+            include rowlabels
 
         Returns
         -------
         np.array
             data in x padded with fill value to original shape of skydata
         """
+        x = np.asarray(x)
         mask = np.copy(self.daymask)
         mask[self.daymask] = self.mask
         px = np.full((self.skydata.shape[0], *x.shape[1:]), fill_value,
-                     dtype=float)
+                     dtype=x.dtype)
         px[mask] = x
+        if rowlabels:
+            px = np.hstack((self.rowlabel, px))
         return px
 
+    def label(self, x):
+        x = np.asarray(x).reshape(self.daysteps, -1)
+        return np.hstack((self.rowlabel[self.daymask], x))
+
     def masked_idx(self, i):
         j = np.searchsorted(self._maskindices, i)
         return j[self._maskindices[j] == i]
 
-    def sky_description(self, i, prefix="skydata", grid=False, sun=True):
+    def sky_description(self, i, prefix="skydata", grid=False, sun=True,
+                        ground=True):
         """generate radiance scene files to directly render sky data at index i
 
         Parameters
@@ -340,12 +401,14 @@ class SkyData(object):
         f.write(f"void brightdata skyfunc 4 {fun} {outf}.dat {outf}.cal bin 0 "
                 "0\nskyfunc glow skyglow 0 0 4 1 1 1 0\n"
                 "skyglow source sky 0 0 4 0 0 1 180\n")
+        if ground:
+            f.write("skyglow source ground 0 0 4 0 0 -1 180\n")
         if sun:
             c = (self.sun[mi, 3], self.sun[mi, 3], self.sun[mi, 3])
             f.write(RadFmt.get_sundef(self.sun[mi, 0:3], c))
         f.close()
         f = open(f"{outf}.cal", 'w')
-        f.write(f"side:{self.skyres};\n{translate.scbinscal}")
+        f.write(f"side:{self.side};\n{translate.scbinscal}")
         f.close()
         data = self.smtx[mi]
         nrbins = data.size
