@@ -1,5 +1,5 @@
 #ifndef lint
-static const char	RCSid[] = "$Id: rtrace.c,v 2.99 2020/07/20 15:54:29 greg Exp $";
+static const char	RCSid[] = "$Id: rtrace.c,v 2.102 2021/04/16 16:31:35 greg Exp $";
 #endif
 /*
  *  rtrace.c - program and variables for individual ray tracing.
@@ -52,6 +52,8 @@ int  castonly = 0;			/* only doing ray-casting? */
 #endif
 OBJECT	traset[MAXTSET+1]={0};		/* trace include/exclude set */
 
+static int  Tflag = 0;			/* source tracing enabled? */
+
 static RAY  thisray;			/* for our convenience */
 
 static FILE  *inpfp = NULL;		/* input stream pointer */
@@ -119,6 +121,16 @@ formstr(				/* return format identifier */
 }
 
 
+static void
+trace_sources(void)			/* trace rays to light sources, also */
+{
+	int	sn;
+
+	for (sn = 0; sn < nsources; sn++)
+		source[sn].sflags |= SFOLLOW;
+}
+
+
 void
 rtrace(				/* trace rays from file */
 	char  *fname,
@@ -148,6 +160,8 @@ rtrace(				/* trace rays from file */
 					/* set up output */
 	if (castonly || every_out[0] != NULL)
 		nproc = 1;		/* don't bother multiprocessing */
+	if (Tflag && every_out[0] != NULL)
+		trace_sources();	/* asking to trace light sources */
 	if ((nextflush > 0) & (nproc > nextflush)) {
 		error(WARNING, "reducing number of processes to match flush interval");
 		nproc = nextflush;
@@ -217,16 +231,6 @@ rtrace(				/* trace rays from file */
 }
 
 
-static void
-trace_sources(void)			/* trace rays to light sources, also */
-{
-	int	sn;
-	
-	for (sn = 0; sn < nsources; sn++)
-		source[sn].sflags |= SFOLLOW;
-}
-
-
 int
 setrtoutput(void)			/* set up output tables, return #comp */
 {
@@ -241,8 +245,7 @@ setrtoutput(void)			/* set up output tables, return #comp */
 	do
 		switch (*vs) {
 		case 'T':				/* trace sources */
-			if (!vs[1]) break;
-			trace_sources();
+			Tflag++;
 			/* fall through */
 		case 't':				/* trace */
 			if (!vs[1]) break;
@@ -420,21 +423,22 @@ rtcompute(			/* compute and print ray value(s) */
 )
 {
 					/* set up ray */
-	rayorigin(&thisray, PRIMARY, NULL, NULL);
 	if (imm_irrad) {
 		VSUM(thisray.rorg, org, dir, 1.1e-4);
 		thisray.rdir[0] = -dir[0];
 		thisray.rdir[1] = -dir[1];
 		thisray.rdir[2] = -dir[2];
 		thisray.rmax = 0.0;
-		thisray.revf = rayirrad;
 	} else {
 		VCOPY(thisray.rorg, org);
 		VCOPY(thisray.rdir, dir);
 		thisray.rmax = dmax;
-		if (castonly)
-			thisray.revf = raycast;
 	}
+	rayorigin(&thisray, PRIMARY, NULL, NULL);
+	if (imm_irrad)
+		thisray.revf = rayirrad;
+	else if (castonly)
+		thisray.revf = raycast;
 	if (ray_pnprocs > 1) {		/* multiprocessing FIFO? */
 		if (ray_fifo_in(&thisray) < 0)
 			error(USER, "lost children");
@@ -765,14 +769,11 @@ static RREAL	vdummy[3] = {0.0, 0.0, 0.0};
 
 
 static void
-oputp(				/* print point */
+oputp(				/* print intersection point */
 	RAY  *r
 )
 {
-	if (r->rot < FHUGE*.99)
-		(*putreal)(r->rop, 3);
-	else
-		(*putreal)(vdummy, 3);
+	(*putreal)(r->rop, 3);	/* set to ray origin if distant or no hit */
 }
 
 
@@ -781,17 +782,18 @@ oputN(				/* print unperturbed normal */
 	RAY  *r
 )
 {
-	if (r->rot < FHUGE*.99) {
-		if (r->rflips & 1) {	/* undo any flippin' flips */
-			FVECT	unrm;
-			unrm[0] = -r->ron[0];
-			unrm[1] = -r->ron[1];
-			unrm[2] = -r->ron[2];
-			(*putreal)(unrm, 3);
-		} else
-			(*putreal)(r->ron, 3);
-	} else
+	if (r->ro == NULL) {	/* zero vector if clipped or no hit */
 		(*putreal)(vdummy, 3);
+		return;
+	}
+	if (r->rflips & 1) {	/* undo any flippin' flips */
+		FVECT	unrm;
+		unrm[0] = -r->ron[0];
+		unrm[1] = -r->ron[1];
+		unrm[2] = -r->ron[2];
+		(*putreal)(unrm, 3);
+	} else
+		(*putreal)(r->ron, 3);
 }
 
 
@@ -802,7 +804,7 @@ oputn(				/* print perturbed normal */
 {
 	FVECT  pnorm;
 
-	if (r->rot >= FHUGE*.99) {
+	if (r->ro == NULL) {	/* clipped or no hit */
 		(*putreal)(vdummy, 3);
 		return;
 	}
