@@ -21,12 +21,12 @@
 #include "rtrace.hh"
 
 namespace ray{
-#include <ray.h>
+#include "Radiance/src/rt/ray.h"
 #include "csrc/rtinit.h"
     extern "C" {
-#include <ambient.h>
-#include <pmapray.h>
-#include <func.h>
+#include "Radiance/src/rt/ambient.h"
+#include "Radiance/src/rt/pmapray.h"
+#include "Radiance/src/rt/func.h"
     }
 }
 
@@ -163,7 +163,7 @@ ncomp: int
 
 
 PYBIND11_MODULE(rtrace_c, m){
-  py::class_<Rtrace>(m, "cRtrace", R"pbdoc(singleton interface to the Radiance rtrace executable.
+  py::class_<Rtrace>(m, "cRtrace", py::module_local(), R"pbdoc(singleton interface to the Radiance rtrace executable.
 
 See the rtrace man page for a full description of the programs functionality. Instance is initialized with a list
 of arguments similar to the command line tool, but with several differences:
